@@ -6,6 +6,29 @@
 #include <genericStack.h>
 #include <genericHash/cloak.h>
 
+/* ====================================================================== */
+/* gcc family has the __builtin_expect that optimizes branch prediction.  */
+/* This is off by default, but you can set the macros                     */
+/* GENERICHASH_LIKELY and GENERICHASH_UNLIKELY to handle that.            */
+/* The use case is to set likely macros only on the inner thing, i.e.     */
+/* genericStack. So the genericHash likely macros by default inherit from */
+/* genericStack likely macros.                                            */
+/* ====================================================================== */
+#ifndef GENERICHASH_LIKELY
+#  ifdef GENERICSTACK_LIKELY
+#    define GENERICHASH_LIKELY(x) GENERICSTACK_LIKELY(x)
+#  else
+#    define GENERICHASH_LIKELY(x) x
+#  endif
+#endif
+#ifndef GENERICHASH_UNLIKELY
+#  ifdef GENERICSTACK_UNLIKELY
+#    define GENERICHASH_UNLIKELY(x) GENERICSTACK_UNLIKELY(x)
+#  else
+#    define GENERICHASH_UNLIKELY(x) x
+#  endif
+#endif
+
 /* A hash is nothing else but a generic stack of generic stacks */
 
 typedef int    (*genericHashKeyIndFunction_t)(void *userDatavp, genericStackItemType_t itemType, void **pp);
@@ -62,11 +85,11 @@ typedef struct genericHash {
     genericHashValCopyFunction_t _valCopyFunctionp = (genericHashValCopyFunction_t) (thisValCopyFunctionp); \
     genericHashValFreeFunction_t _valFreeFunctionp = (genericHashValFreeFunction_t) (thisValFreeFunctionp); \
     									\
-    if (_keyIndFunctionp == NULL) {                                     \
+    if (GENERICHASH_UNLIKELY(_keyIndFunctionp == NULL)) {               \
       hashName = NULL;                                                  \
     } else {                                                            \
-      hashName = malloc(sizeof(genericHash_t));				\
-      if (hashName != NULL) {						\
+      hashName = (genericHash_t *) malloc(sizeof(genericHash_t));	\
+      if (GENERICHASH_LIKELY(hashName != NULL)) {                       \
         hashName->wantedSubSize    = (thisWantedSubSize);               \
         hashName->keyIndFunctionp  = _keyIndFunctionp;			\
         hashName->keyCmpFunctionp  = _keyCmpFunctionp;			\
@@ -106,7 +129,7 @@ typedef struct genericHash {
     hashName->error = 0;						\
     hashName->used = 0;							\
     									\
-    if (_keyIndFunctionp == NULL) {                                     \
+    if (GENERICHASH_UNLIKELY(_keyIndFunctionp == NULL)) {               \
       hashName->error = 1;						\
     } else {                                                            \
       hashName->wantedSubSize    = (thisWantedSubSize);			\
@@ -152,7 +175,7 @@ typedef struct genericHash {
       if (hashName->keyCopyFunctionp != NULL) {                         \
         GENERICSTACKITEMTYPE2TYPE_##keyType _keyValForCopy = (keyVal);  \
         GENERICSTACKITEMTYPE2TYPE_##keyType _p = hashName->keyCopyFunctionp((void *) userDatavp, &_keyValForCopy); \
-        if ((_keyValForCopy != NULL) && (_p == NULL)) {			\
+        if (GENERICHASH_UNLIKELY((_keyValForCopy != NULL) && (_p == NULL))) { \
           hashName->error = 1;						\
         } else {                                                        \
           keyValCopy = _p;                                              \
@@ -167,7 +190,7 @@ typedef struct genericHash {
       if (hashName->valCopyFunctionp != NULL) {                         \
         GENERICSTACKITEMTYPE2TYPE_##valType _valValForCopy = (valVal);  \
         GENERICSTACKITEMTYPE2TYPE_##valType _p = hashName->valCopyFunctionp((void *) userDatavp, &_valValForCopy); \
-        if ((_valValForCopy != NULL) && (_p == NULL)) {                 \
+        if (GENERICHASH_UNLIKELY((_valValForCopy != NULL) && (_p == NULL))) { \
           hashName->error = 1;						\
         } else {                                                        \
           valValCopy = _p;                                              \
@@ -188,7 +211,7 @@ typedef struct genericHash {
     GENERICSTACKITEMTYPE2TYPE_##valType _valValCopy;			\
 									\
     _GENERICHASH_COPY(hashName, userDatavp, keyType, keyVal, _keyValCopy, valType, valVal, _valValCopy); \
-    if (hashName->error == 0) {						\
+    if (GENERICHASH_LIKELY(hashName->error == 0)) {                     \
       GENERICSTACK_PUSH_##keyType(subKeyStackp, _keyValCopy);		\
       GENERICSTACK_PUSH_##valType(subValStackp, _valValCopy);		\
       if (GENERICSTACK_ERROR(subKeyStackp) || GENERICSTACK_ERROR(subValStackp)) { \
@@ -207,7 +230,7 @@ typedef struct genericHash {
     GENERICSTACKITEMTYPE2TYPE_##valType _valValCopy;			\
 									\
     _GENERICHASH_COPY(hashName, userDatavp, keyType, keyVal, _keyValCopy, valType, valVal, _valValCopy); \
-    if (hashName->error == 0) {						\
+    if (GENERICHASH_LIKELY(hashName->error == 0)) {                     \
       GENERICSTACK_SET_##keyType(subKeyStackp, _keyValCopy, index);	\
       GENERICSTACK_SET_##valType(subValStackp, _valValCopy, index);	\
       if (GENERICSTACK_ERROR(subKeyStackp) || GENERICSTACK_ERROR(subValStackp)) { \
@@ -230,7 +253,7 @@ typedef struct genericHash {
     GENERICSTACKITEMTYPE2TYPE_##keyType _keyVal = (GENERICSTACKITEMTYPE2TYPE_##keyType) (keyVal); \
     hashName->error = 0;						\
 									\
-    if (subStackIndex < 0 ) {                                           \
+    if (GENERICHASH_UNLIKELY(subStackIndex < 0)) {                      \
       errno = EINVAL;							\
       hashName->error = 1;						\
     } else {								\
@@ -266,8 +289,6 @@ typedef struct genericHash {
 	genericStack_t *_subValStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->valStackp, subStackIndex); \
 	int             _subStackusedi = GENERICSTACK_USED(_subKeyStackp); \
 	int             _i;						\
-        short           _keyCmpFunctionb = (hashName->keyCmpFunctionp != NULL); \
-        short           _keyFreeFunctionb = (hashName->keyFreeFunctionp != NULL); \
         short           _valFreeFunctionb = (hashName->valFreeFunctionp != NULL); \
 									\
 	for (_i = 0; _i < _subStackusedi; _i++) {			\
@@ -278,14 +299,17 @@ typedef struct genericHash {
 									\
 	  _gotKeyVal = GENERICSTACK_GET_##keyType(_subKeyStackp, _i);	\
           GENERICHASH_IIF(GENERICHASH_EQUAL(keyType, PTR)) (            \
-            if (_keyCmpFunctionb) {                                     \
-              if (! hashName->keyCmpFunctionp((void *) userDatavp, &_keyVal, &_gotKeyVal)) { \
-                continue;                                               \
-              }								\
-            } else {							\
-              if (_keyVal != _gotKeyVal) {				\
-                continue;                                               \
-              }								\
+            {                                                           \
+              short _keyCmpFunctionb = (hashName->keyCmpFunctionp != NULL); \
+              if (_keyCmpFunctionb) {                                   \
+                if (! hashName->keyCmpFunctionp((void *) userDatavp, &_keyVal, &_gotKeyVal)) { \
+                  continue;                                             \
+                }                                                       \
+              } else {							\
+                if (_keyVal != _gotKeyVal) {				\
+                  continue;                                             \
+                }                                                       \
+              }                                                         \
             }                                                           \
             ,                                                           \
 	    if (_keyVal != _gotKeyVal) {				\
@@ -294,8 +318,11 @@ typedef struct genericHash {
           )                                                             \
 									\
           GENERICHASH_IIF(GENERICHASH_EQUAL(keyType, PTR)) (            \
-            if (_keyFreeFunctionb && (_gotKeyVal != NULL)) {            \
-              hashName->keyFreeFunctionp((void *) userDatavp, &_gotKeyVal); \
+            {                                                           \
+              short _keyFreeFunctionb = (hashName->keyFreeFunctionp != NULL); \
+              if (_keyFreeFunctionb && (_gotKeyVal != NULL)) {          \
+                hashName->keyFreeFunctionp((void *) userDatavp, &_gotKeyVal); \
+              }								\
             }								\
             ,                                                           \
           )                                                             \
@@ -321,7 +348,7 @@ typedef struct genericHash {
 /* ====================================================================== */
 #define _GENERICHASH_FIND_REMOVE(hashName, userDatavp, keyType, keyVal, valType, valValp, findResult, remove) do { \
 									\
-    if (hashName->keyIndFunctionp == NULL) {				\
+    if (GENERICHASH_UNLIKELY(hashName->keyIndFunctionp == NULL)) {      \
       errno = EINVAL;							\
       hashName->error = 1;						\
     } else {								\
@@ -332,7 +359,7 @@ typedef struct genericHash {
 
 #define _GENERICHASH_FIND_REMOVE_BY_IND(hashName, userDatavp, keyType, keyVal, valType, valValp, findResult, _subStackIndex, remove) do { \
 									\
-    if (_subStackIndex < 0) {						\
+    if (GENERICHASH_UNLIKELY(_subStackIndex < 0)) {                     \
       errno = EINVAL;							\
       hashName->error = 1;						\
     } else {								\
@@ -344,8 +371,6 @@ typedef struct genericHash {
 	genericStack_t *_subKeyStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->keyStackp, _subStackIndex); \
 	genericStack_t *_subValStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->valStackp, _subStackIndex); \
 	int             _subStackusedi = GENERICSTACK_USED(_subKeyStackp); \
-        short           _keyCmpFunctionb = (hashName->keyCmpFunctionp != NULL); \
-        short           _keyFreeFunctionb = (hashName->keyFreeFunctionp != NULL); \
         short           _valFreeFunctionb = (hashName->valFreeFunctionp != NULL); \
 	int _i;								\
 									\
@@ -358,14 +383,17 @@ typedef struct genericHash {
 									\
 	  _gotKeyVal = GENERICSTACK_GET_##keyType(_subKeyStackp, _i);	\
           GENERICHASH_IIF(GENERICHASH_EQUAL(keyType, PTR)) (            \
-            if (_keyCmpFunctionb && (GENERICSTACKITEMTYPE_##keyType == GENERICSTACKITEMTYPE_PTR)) { \
-              if (! hashName->keyCmpFunctionp((void *) userDatavp, (void **) &keyVal, (void **) &_gotKeyVal)) { \
-                continue;                                               \
-              }								\
-            } else {							\
-              if (keyVal != _gotKeyVal) {                               \
-                continue;                                               \
-              }								\
+            {                                                           \
+              short _keyCmpFunctionb = (hashName->keyCmpFunctionp != NULL); \
+              if (_keyCmpFunctionb && (GENERICSTACKITEMTYPE_##keyType == GENERICSTACKITEMTYPE_PTR)) { \
+                if (! hashName->keyCmpFunctionp((void *) userDatavp, (void **) &keyVal, (void **) &_gotKeyVal)) { \
+                  continue;                                             \
+                }                                                       \
+              } else {							\
+                if (keyVal != _gotKeyVal) {                             \
+                  continue;                                             \
+                }                                                       \
+              }                                                         \
             }                                                           \
             ,                                                           \
 	    if (keyVal != _gotKeyVal) {					\
@@ -380,9 +408,12 @@ typedef struct genericHash {
 	  }								\
 	  if (remove) {							\
             GENERICHASH_IIF(GENERICHASH_EQUAL(keyType, PTR)) (          \
-	      if (_keyFreeFunctionb && (_gotKeyVal != NULL)) {          \
-                hashName->keyFreeFunctionp((void *) userDatavp, &_gotKeyVal); \
-              }								\
+	      {                                                         \
+                short _keyFreeFunctionb = (hashName->keyFreeFunctionp != NULL); \
+                if (_keyFreeFunctionb && (_gotKeyVal != NULL)) {        \
+                  hashName->keyFreeFunctionp((void *) userDatavp, &_gotKeyVal); \
+                }                                                       \
+              }                                                         \
               ,                                                         \
             )                                                           \
 	    GENERICSTACK_SET_NA(_subKeyStackp, _i);			\
@@ -414,7 +445,9 @@ typedef struct genericHash {
 #define GENERICHASH_FIND_BY_IND(hashName, userDatavp, keyType, keyVal, valType, valValp, findResult, index) _GENERICHASH_FIND_REMOVE_BY_IND(hashName, userDatavp, keyType, keyVal, valType, valValp, findResult, index, 0)
 #define GENERICHASH_REMOVE_BY_IND(hashName, userDatavp, keyType, keyVal, valType, valValp, findResult, index) _GENERICHASH_FIND_REMOVE_BY_IND(hashName, userDatavp, keyType, keyVal, valType, valValp, findResult, index, 1)
 
-#define GENERICHASH_RESET(hashName, userDatavp) do {                    \
+/* Take care: keyStackp and valStackp are always inside hash structure, i.e. this is a RESET for them, even in FREE mode, never a RELAX */
+/* .........: subStack's are always on the stack: i.e. this is a FREE for them, even in RESET mode, RELAX is supported */
+#define _GENERICHASH_DISPOSE(hashName, userDatavp, freeb, relaxb, resetb) do { \
     int _usedi  = GENERICSTACK_USED(hashName->keyStackp);		\
     int _i;								\
     short _keyFreeFunctionb = (hashName->keyFreeFunctionp != NULL);     \
@@ -443,75 +476,36 @@ typedef struct genericHash {
               }								\
             }								\
           }								\
-          GENERICSTACK_FREE(_subKeyStackp);				\
-          GENERICSTACK_FREE(_subValStackp);				\
+          if (freeb || resetb) {                                        \
+            GENERICSTACK_FREE(_subKeyStackp);                          \
+            GENERICSTACK_FREE(_subValStackp);                           \
+          } else if (relaxb) {                                          \
+            GENERICSTACK_RELAX(_subKeyStackp);                          \
+            GENERICSTACK_RELAX(_subValStackp);                          \
+          }                                                             \
         }                                                               \
-      }                                                                 \
+      }									\
     } else {                                                            \
-      /* GENERICSTACK_FREE is NULL protected */                         \
       for (_i = 0; _i < _usedi; _i++) {					\
         if ((GENERICSTACKITEMTYPE(hashName->keyStackp, _i) == GENERICSTACKITEMTYPE_PTR) && \
             (GENERICSTACKITEMTYPE(hashName->valStackp, _i) == GENERICSTACKITEMTYPE_PTR)) { \
           genericStack_t *_subKeyStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->keyStackp, _i); \
           genericStack_t *_subValStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->valStackp, _i); \
                                                                         \
-          GENERICSTACK_FREE(_subKeyStackp);                             \
-          GENERICSTACK_FREE(_subValStackp);				\
+          if (freeb || resetb) {                                        \
+            GENERICSTACK_FREE(_subKeyStackp);                           \
+            GENERICSTACK_FREE(_subValStackp);                           \
+          } else if (relaxb) {                                          \
+            GENERICSTACK_RELAX(_subKeyStackp);                          \
+            GENERICSTACK_RELAX(_subValStackp);                          \
+          }                                                             \
         }                                                               \
-      }                                                                 \
+      }									\
     }                                                                   \
-    GENERICSTACK_RESET(hashName->keyStackp);				\
-    GENERICSTACK_RESET(hashName->valStackp);				\
-    									\
-    hashName->error = 0;						\
-    hashName->used = 0;							\
-  } while (0)
-
-#define GENERICHASH_RELAX(hashName, userDatavp) do {                    \
-    int _usedi  = GENERICSTACK_USED(hashName->keyStackp);		\
-    int _i;								\
-    short _keyFreeFunctionb = (hashName->keyFreeFunctionp != NULL);     \
-    short _valFreeFunctionb = (hashName->valFreeFunctionp != NULL);     \
                                                                         \
-    if (_keyFreeFunctionb || _valFreeFunctionb) {                       \
-      for (_i = 0; _i < _usedi; _i++) {					\
-        if ((GENERICSTACKITEMTYPE(hashName->keyStackp, _i) == GENERICSTACKITEMTYPE_PTR) && \
-            (GENERICSTACKITEMTYPE(hashName->valStackp, _i) == GENERICSTACKITEMTYPE_PTR)) { \
-          genericStack_t *_subKeyStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->keyStackp, _i); \
-          genericStack_t *_subValStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->valStackp, _i); \
-          int             _subStackusedi = GENERICSTACK_USED(_subKeyStackp); \
-          int             _j;						\
-                                                                        \
-          for (_j = 0; _j < _subStackusedi; _j++) {			\
-            if (_keyFreeFunctionb && (GENERICSTACKITEMTYPE(_subKeyStackp, _j) == GENERICSTACKITEMTYPE_PTR)) { \
-              GENERICSTACKITEMTYPE2TYPE_PTR _keyValp = GENERICSTACK_GET_PTR(_subKeyStackp, _j); \
-              if (_keyValp != NULL) {					\
-                hashName->keyFreeFunctionp(userDatavp, &_keyValp);      \
-              }								\
-            }								\
-            if (_valFreeFunctionb && (GENERICSTACKITEMTYPE(_subValStackp, _j) == GENERICSTACKITEMTYPE_PTR)) { \
-              GENERICSTACKITEMTYPE2TYPE_PTR _valValp = GENERICSTACK_GET_PTR(_subValStackp, _j); \
-              if (_valValp != NULL) {					\
-                hashName->valFreeFunctionp(userDatavp, &_valValp);      \
-              }								\
-            }								\
-          }								\
-          GENERICSTACK_RESET(_subKeyStackp);                            \
-          GENERICSTACK_RESET(_subValStackp);                            \
-        }                                                               \
-      }									\
-    } else {                                                            \
-      /* GENERICSTACK_RESET is NULL protected */                        \
-      for (_i = 0; _i < _usedi; _i++) {					\
-        if ((GENERICSTACKITEMTYPE(hashName->keyStackp, _i) == GENERICSTACKITEMTYPE_PTR) && \
-            (GENERICSTACKITEMTYPE(hashName->valStackp, _i) == GENERICSTACKITEMTYPE_PTR)) { \
-          genericStack_t *_subKeyStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->keyStackp, _i); \
-          genericStack_t *_subValStackp = (genericStack_t *) GENERICSTACK_GET_PTR(hashName->valStackp, _i); \
-                                                                        \
-          GENERICSTACK_RESET(_subKeyStackp);                            \
-          GENERICSTACK_RESET(_subValStackp);                            \
-        }                                                               \
-      }									\
+    if (freeb || resetb) {                                              \
+      GENERICSTACK_RESET(hashName->keyStackp);                          \
+      GENERICSTACK_RESET(hashName->valStackp);                          \
     }                                                                   \
                                                                         \
     hashName->error = 0;                                                \
@@ -519,11 +513,15 @@ typedef struct genericHash {
   } while (0)
 
 #define GENERICHASH_FREE(hashName, userDatavp) do {                     \
-    if (hashName != NULL) {						\
-      GENERICHASH_RESET(hashName, userDatavp);				\
+    if (GENERICHASH_LIKELY(hashName != NULL)) {                         \
+      _GENERICHASH_DISPOSE(hashName, userDatavp, 1 /* freeb */, 0 /* relaxb */, 0 /* resetb */); \
       free(hashName);							\
       hashName = NULL;							\
     }									\
   } while (0)
 
+#define GENERICHASH_RELAX(hashName, userDatavp) _GENERICHASH_DISPOSE(hashName, userDatavp, 0 /* freeb */, 1 /* relaxb */, 0 /* resetb */)
+
+#define GENERICHASH_RESET(hashName, userDatavp) _GENERICHASH_DISPOSE(hashName, userDatavp, 0 /* freeb */, 0 /* relaxb */, 1 /* resetb */)
+
 #endif /* GENERICHASH_H */
